@@ -10,9 +10,11 @@
 
 __version__ = "0.5.2"
 
+import functools
 import inspect
-from flask import Response, make_response
-import re, copy
+from werkzeug.routing import parse_rule
+from flask import request, Response, make_response
+import re
 
 _temp_rule_cache = None
 
@@ -50,6 +52,9 @@ class FlaskView(object):
 
     __metaclass__ = _FlaskViewMeta
 
+    #: A list of decorators to be applied to all view of this class
+    decorators = []
+
     @classmethod
     def register(cls, app, route_base=None, subdomain=None):
         """Registers a FlaskView class for use with a specific instance of a
@@ -77,7 +82,7 @@ class FlaskView(object):
             cls.route_base = route_base
 
         if not subdomain:
-            if hasattr(app, "subdomain") and app.subdomain != None:
+            if hasattr(app, "subdomain") and app.subdomain is not None:
                 subdomain = app.subdomain
             elif hasattr(cls, "subdomain"):
                 subdomain = cls.subdomain
@@ -88,38 +93,40 @@ class FlaskView(object):
         for name, value in members:
             proxy = cls.make_proxy_method(name)
             route_name = cls.build_route_name(name)
+            try:
+                if hasattr(cls, "_rule_cache") and name in cls._rule_cache:
+                    for idx, cached_rule in enumerate(cls._rule_cache[name]):
+                        rule, options = cached_rule
+                        rule = cls.build_rule(rule)
+                        sub, ep, options = cls.parse_options(options)
 
-            if hasattr(cls, "_rule_cache") and name in cls._rule_cache:
-                for idx, cached_rule in enumerate(cls._rule_cache[name]):
-                    rule, options = cached_rule
-                    rule = cls.build_rule(rule)
-                    sub, ep, options = cls.parse_options(options)
+                        if not subdomain and sub:
+                            subdomain = sub
 
-                    if not subdomain and sub:
-                        subdomain = sub
+                        if ep:
+                            endpoint = ep
+                        elif len(cls._rule_cache[name]) == 1:
+                            endpoint = route_name
+                        else:
+                            endpoint = "%s_%d" % (route_name, idx,)
 
-                    if ep:
-                        endpoint = ep
-                    elif len(cls._rule_cache[name]) == 1:
-                        endpoint = route_name
+                        app.add_url_rule(rule, endpoint, proxy, subdomain=subdomain, **options)
+
+                elif name in special_methods:
+                    if name in ["get", "index"]:
+                        methods = ["GET"]
                     else:
-                        endpoint = "%s_%d" % (route_name, idx,)
+                        methods = [name.upper()]
 
-                    app.add_url_rule(rule, endpoint, proxy, subdomain=subdomain, **options)
+                    rule = cls.build_rule("/", value)
+
+                    app.add_url_rule(rule, route_name, proxy, methods=methods, subdomain=subdomain)
 
-            elif name in special_methods:
-                if name in ["get", "index"]:
-                    methods = ["GET"]
                 else:
-                    methods = [name.upper()]
-
-                rule = cls.build_rule("/", value)
-
-                app.add_url_rule(rule, route_name, proxy, methods=methods, subdomain=subdomain)
-
-            else:
-                rule = cls.build_rule('/%s/' % name, value,)
-                app.add_url_rule(rule, route_name, proxy, subdomain=subdomain)
+                    rule = cls.build_rule('/%s/' % name, value,)
+                    app.add_url_rule(rule, route_name, proxy, subdomain=subdomain)
+            except DecoratorCompatibilityError:
+                raise DecoratorCompatibilityError("Incompatible decorator detected on %s in class %s" % name, cls.__name__)
 
         if hasattr(cls, "orig_route_base"):
             cls.route_base = cls.orig_route_base
@@ -132,7 +139,7 @@ class FlaskView(object):
         """Extracts subdomain and endpoint values from the options dict and returns
            them along with a new dict without those values.
         """
-        options = copy.copy(options)
+        options = options.copy()
         subdomain = options.pop('subdomain', None)
         endpoint = options.pop('endpoint', None)
         return subdomain, endpoint, options,
@@ -147,18 +154,29 @@ class FlaskView(object):
         :param name: the name of the method to create a proxy for
         """
 
-        def proxy(*args, **kwargs):
-            i = cls()
+        i = cls()
+        view = getattr(i, name)
+
+        if cls.decorators:
+            for decorator in cls.decorators:
+                view = decorator(view)
+
+        @functools.wraps(view)
+        def proxy(**forgettable_view_args):
+            # Always use the global request object's view_args, because they
+            # can be modified by intervening function before an endpoint or
+            # wrapper gets called. This matches Flask's behavior.
+            del forgettable_view_args
+
             if hasattr(i, "before_request"):
-                i.before_request(name, *args, **kwargs)
+                i.before_request(name, **request.view_args)
 
             before_view_name = "before_" + name
             if hasattr(i, before_view_name):
                 before_view = getattr(i, before_view_name)
-                before_view(*args, **kwargs)
+                before_view(**request.view_args)
 
-            view = getattr(i, name)
-            response = view(*args, **kwargs)
+            response = view(**request.view_args)
             if not isinstance(response, Response):
                 response = make_response(response)
 
@@ -182,6 +200,7 @@ class FlaskView(object):
         all_members = inspect.getmembers(cls, predicate=inspect.ismethod)
         return [member for member in all_members
                 if not member[0] in base_members
+                and not member[1].im_self == cls
                 and not member[0].startswith("_")
                 and not member[0].startswith("before_")
                 and not member[0].startswith("after_")]
@@ -201,11 +220,14 @@ class FlaskView(object):
 
         route_base = cls.get_route_base()
         rule_parts = [route_base, rule]
+        ignored_rule_args = ['self']
+        if hasattr(cls, 'base_args'):
+            ignored_rule_args += cls.base_args
 
         if method:
-            args = inspect.getargspec(method)[0]
+            args = get_true_argspec(method)[0]
             for arg in args:
-                if arg != "self":
+                if arg not in ignored_rule_args:
                     rule_parts.append("<%s>" % arg)
 
         result = "/%s" % "/".join(rule_parts)
@@ -217,6 +239,8 @@ class FlaskView(object):
 
         if hasattr(cls, "route_base"):
             route_base = cls.route_base
+            base_rule = parse_rule(route_base)
+            cls.base_args = [r[2] for r in base_rule]
         else:
             if cls.__name__.endswith("View"):
                 route_base = cls.__name__[:-4].lower()
@@ -236,4 +260,27 @@ class FlaskView(object):
         return cls.__name__ + ":%s" % method_name
 
 
+def get_true_argspec(method):
+    """Drills through layers of decorators attempting to locate the actual argspec for the method."""
+
+    argspec = inspect.getargspec(method)
+    args = argspec[0]
+    if args and args[0] == 'self':
+        return argspec
+    if hasattr(method, '__func__'):
+        method = method.__func__
+    if not hasattr(method, 'func_closure') or method.func_closure is None:
+        raise DecoratorCompatibilityError
+
+    method = method.func_closure[0].cell_contents
+    return get_true_argspec(method)
+
+
+class DecoratorCompatibilityError(Exception):
+    pass
+
+
+
+
+
 
